from decouple import AutoConfig, Csv

config = AutoConfig()


# Github Config
GITHUB_DEFAULT_PAGE_SIZE = config("GITHUB_DEFAULT_PAGE_SIZE", cast=int, default=100)
GITHUB_ORG = config("GITHUB_ORG", cast=str)
GITHUB_REPOSITORIES = config("GITHUB_REPOSITORIES", cast=Csv(post_process=list[str], strip=" "))
GITHUB_TOKEN = config("GITHUB_TOKEN", cast=str)
GITHUB_URL = config("GITHUB_URL", cast=str, default="https://api.github.com")
GITHUB_USER = config("GITHUB_USER", cast=str)

# External Requests Config
EXTERNAL_SERVICE_MAXIMUM_REQUEST_ATTEMPTS = config("EXTERNAL_SERVICE_MAXIMUM_REQUEST_ATTEMPTS", cast=int, default=3)
EXTERNAL_SERVICE_WAIT_IN_MS_BETWEEN_REQUESTS = config(
    "EXTERNAL_SERVICE_WAIT_IN_MS_BETWEEN_REQUESTS", cast=int, default=250
)

# Deps Config
DEPS_EXPORT_TO_SVG = config("DEPS_EXPORT_TO_SVG", cast=bool, default=False)



from dataclasses import dataclass
from http import HTTPStatus

import requests
from requests.exceptions import ConnectionError as HTTPConnectionError, HTTPError
from retrying import retry

from deps.config import EXTERNAL_SERVICE_MAXIMUM_REQUEST_ATTEMPTS, EXTERNAL_SERVICE_WAIT_IN_MS_BETWEEN_REQUESTS


def retry_if_connection_or_http_error(exception: Exception) -> bool:
    """Retries on connection and HTTP errors"""
    if isinstance(exception, (HTTPConnectionError, HTTPError)):
        return True

    return False


@dataclass
class PackageCache:
    """A cache of the latest versions of packages"""

    cache: dict[str, dict[str, str]]

    def __init__(self) -> None:
        """Initialize the cache"""
        self.cache: dict[str, str] = {}

    @retry(
        retry_on_exception=retry_if_connection_or_http_error,
        stop_max_attempt_number=EXTERNAL_SERVICE_MAXIMUM_REQUEST_ATTEMPTS,
        wait_exponential_multiplier=EXTERNAL_SERVICE_WAIT_IN_MS_BETWEEN_REQUESTS,
    )
    def retrieve(self, name: str) -> None:
        """Retrieve the latest version of a package from PyPI"""
        response = requests.get(f"https://pypi.org/pypi/{name}/json", timeout=10)

        if response.status_code is not HTTPStatus.OK:
            response.raise_for_status()

        if info := response.json()["info"]:
            self.cache[name] = {
                "available_version": str(info["version"]),
                "release_url": str(info["release_url"]),
            }

    def get(self, name: str) -> dict[str, str]:
        """Get the latest version of a package from the cache"""
        if name not in self.cache:
            self.retrieve(name)

        return self.cache[name]


cache: PackageCache = PackageCache()



from dataclasses import dataclass
from re import match
from typing import Optional

from requests import get
from requests.auth import HTTPBasicAuth

from deps.config import GITHUB_ORG, GITHUB_TOKEN, GITHUB_USER
from deps.storage import cache

REGEX_PIPENV_DEPENDENCY_LINE = r"^(?P<name>.*)\s*\=\s*\"\=\=?(?P<version>.*)\"$"
REGEX_PIP_DEPENDENCY_LINE = r"^(?P<name>.*)==(?P<version>.*)$"


@dataclass
class DependenciesResolver:
    """Resolves package versions from Pipfile against PyPI"""

    versions_by_service: dict
    auth = None

    def __init__(self) -> None:
        """Initialize the resolver"""
        self.versions_by_service = {}
        self.auth = HTTPBasicAuth(GITHUB_USER, GITHUB_TOKEN)

    def retrieve_file(self, repo: str) -> str:
        """Retrieve the Pipfile from GitHub"""
        urls = [
            f"https://raw.githubusercontent.com/{GITHUB_ORG}/{repo}/main/Pipfile",
            f"https://raw.githubusercontent.com/{GITHUB_ORG}/{repo}/master/Pipfile",
            f"https://raw.githubusercontent.com/{GITHUB_ORG}/{repo}/main/requirements.txt",
            f"https://raw.githubusercontent.com/{GITHUB_ORG}/{repo}/master/requirements.txt",
        ]

        for url in urls:
            response = get(url=url, auth=self.auth, timeout=10)

            if response.status_code != 404:
                info: str = response.content.decode(encoding="UTF8")

                return info

        return ""

    def compare_package_versions(self, repo: str, lines: str) -> dict:
        """Compare the package versions"""
        if not lines:
            return {}

        for line in lines.split("\n"):
            name: Optional[str] = None
            version: Optional[str] = None

            if pypi_match := match(REGEX_PIPENV_DEPENDENCY_LINE, line):
                name = pypi_match["name"].strip()
                version = pypi_match["version"].strip()
            elif pypi_match := match(REGEX_PIP_DEPENDENCY_LINE, line):
                name = pypi_match["name"].strip()
                version = pypi_match["version"].strip()
            else:
                # dependency does not match either pipfile or requirements.txt - skipping
                continue

            # replace invalid characters in the name
            name = name.replace("=", "")

            if repo not in self.versions_by_service:
                self.versions_by_service[repo] = []

            info: dict[str, str] = cache.get(name=name)
            available_version: str = info["available_version"]
            release_url: str = info["release_url"]

            self.versions_by_service[repo].append(
                {
                    "dependency_name": name,
                    "current_version": version,
                    "available_version": available_version,
                    "release_url": release_url,
                }
            )

        return self.versions_by_service



from setuptools import find_namespace_packages, setup


def _requires_from_file(filename):
    with open(filename) as f:
        return f.read().splitlines()


with open("README.md") as fh:
    long_description = fh.read()


setup(
    name="deps",
    packages=find_namespace_packages(include=["*"]),
    version="0.2.0",
    license="MIT",
    description=("A terminal UI dashboard to view python dependencies across Github repositories."),
    author="Kyle Harrison",
    author_email="kyle.harrison.dev@gmail.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    download_url="https://pypi.org/project/deps/",
    project_urls={
        "Funding": "https://ko-fi.com/apoclyps",
        "Say Thanks!": "https://twitter.com/apoclyps",
        "Source": "https://github.com/apoclyps/deps",
        "Tracker": "https://github.com/apoclyps/deps/issues",
    },
    keywords=["Deps"],
    install_requires=_requires_from_file("requirements.txt"),
    entry_points={"console_scripts": ["deps = deps.cli.main:main"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Terminals",
    ],
)
